window.Gleam.initSearch([{"doc":"event_hub","title":"event_hub","content":"# Event-Hub\n\n[![Package Version](https://img.shields.io/hexpm/v/event_hub)](https://hex.pm/packages/event_hub)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/event_hub/)\n\nEvent-Hub is a Gleam library that provides simple hubs with publishers and\nsubscribers for event-driven observers. It supports asynchronous message\nhandling and event notifications, decoupling components efficiently. It works on\nErlang and JavaScript.\n\n---\n\nFurther documentation can be found at &lt;https://hexdocs.pm/event_hub&gt;.\n\n- [event_hub](https://hexdocs.pm/event_hub/event_hub.html)\n- [event_hub/filtered](https://hexdocs.pm/event_hub/event_hub/filtered.html)\n- [event_hub/reactive](https://hexdocs.pm/event_hub/event_hub/reactive.html)\n- [event_hub/stateful](https://hexdocs.pm/event_hub/event_hub/stateful.html)\n- [event_hub/topic](https://hexdocs.pm/event_hub/event_hub/topic.html)\n\n## Try it yourself!\n\n```sh\ngleam add event_hub\n```\n\n## Examples\n\n### Imports:\n\nThe following imports where used for the examples:\n\n```gleam\nimport gleam/int\nimport gleam/io\nimport event_hub\nimport event_hub/filtered\nimport event_hub/reactive\nimport event_hub/stateful\nimport event_hub/topic\n```\n\n### Simple Observer\n\nA simple observer implementation. This example demonstrates the basic usage of\nthe Event-Hub library. It is an easy way to use publishers and subscribers to\nhandle events.\n\n```gleam\nfn simple_observer() {\n  // Creates a new hub for distributing events.\n  use hub &lt;- event_hub.new()\n\n  // Notifies all subscribers of the hub that an event has occurred.\n  event_hub.notify(hub, 1)\n\n  // You can forward the hub to other functions or components.\n  {\n    // Using syntactic sugar for handling the callback.\n    use value &lt;- event_hub.subscribe(hub)\n\n    // This function gets called when the hub receives an event.\n    io.println(&quot;[1] | Received an event with value: &quot; &lt;&gt; int.to_string(value))\n  }\n\n  // You can also subscribe using a normal callback function.\n  // This also returns an unsubscribe function.\n  let unsubscribe =\n    event_hub.subscribe(hub, fn(value) {\n      io.println(&quot;[2] | Received an event with value: &quot; &lt;&gt; int.to_string(value))\n    })\n\n  // Notifies all subscribers of the hub that an event has occurred with the value `2`.\n  // These notifications occur in parallel but notify waits for all of them to complete.\n  event_hub.notify(hub, 2)\n\n  // Unsubscribe if you no longer need to receive events.\n  unsubscribe()\n\n  // Notify again to demonstrate that the unsubscribe function works.\n  event_hub.notify(hub, 3)\n}\n```\n\n#### Output:\n\n&gt; [1] | Received an event with value: 2  \n&gt; [2] | Received an event with value: 2  \n&gt; [1] | Received an event with value: 3\n\n### Simple Observer with State\n\nA simple stateful observer implementation. This is like the previous example,\nbut it uses a stateful event_hub.\n\n```gleam\nfn simple_observer_with_state() {\n  // Creates a new hub for distributing events with an initial state.\n  use hub &lt;- stateful.new(&quot;initial state&quot;)\n\n  // Gets the current state.\n  let current_state = stateful.state(hub)\n  io.println(&quot;Current state: &quot; &lt;&gt; current_state)\n\n  // Notifies all subscribers of the hub that an event has occurred.\n  stateful.notify(hub, &quot;test value&quot;)\n\n  // See that the state has changed.\n  let current_state = stateful.state(hub)\n  io.println(&quot;Current state: &quot; &lt;&gt; current_state)\n\n  // You can forward the hub to other functions or components.\n  {\n    // Using syntactic sugar for handling the callback.\n    // With the second argument, you can specify whether the callback should be called with the current state.\n    use value &lt;- stateful.subscribe(hub, True)\n\n    // This function gets called when the hub receives an event.\n    // In this case, it prints the current state because the second argument was `True`.\n    io.println(&quot;[1] | Received an event with value: &quot; &lt;&gt; value)\n  }\n\n  // You can also subscribe using a normal callback function.\n  // This also returns a tuple with the current state and an unsubscribe function.\n  // The second argument specifies that the callback should not be called with the current state.\n  let #(current_state, unsubscribe) =\n    stateful.subscribe(hub, False, fn(value) {\n      io.println(&quot;[2] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // See that the state is still the same.\n  io.println(&quot;Current state: &quot; &lt;&gt; current_state)\n\n  // Notifies all subscribers of the hub that an event has occurred with the value `&quot;third event&quot;`.\n  // These notifications occur in parallel but notify waits for all of them to complete.\n  stateful.notify(hub, &quot;third event&quot;)\n\n  // Unsubscribe if you no longer need to receive events.\n  unsubscribe()\n\n  // Notify again to demonstrate that the unsubscribe function works.\n  stateful.notify(hub, &quot;last one&quot;)\n}\n```\n\n#### Output:\n\n&gt; Current state: initial state  \n&gt; Current state: test value  \n&gt; [1] | Received an event with value: test val  \n&gt; Current state: test value  \n&gt; [1] | Received an event with value: third ev  \n&gt; [2] | Received an event with value: third ev  \n&gt; [1] | Received an event with value: last one\n\n### Reactive Observer\n\nA simple reactive observer implementation. This can be used to broadcast things\nthat constantly change like the current time to clients. Another use case would\nbe to broadcast database updates to listeners.\n\n```gleam\nfn reactive_observer() {\n  // This is a mockup of the current time function.\n  // In a real application, you would use a real time function.\n  let now = fn() { &quot;2024-05-21T16:30:00Z&quot; }\n\n  // Creates a new hub for distributing events with a current time function.\n  use time_hub &lt;- reactive.new(now)\n\n  // Subscribes to the hub to get notified when the current time changes.\n  let unsubscribe_a =\n    reactive.subscribe(time_hub, fn(value) {\n      io.println(&quot;[A] | Current time: &quot; &lt;&gt; value)\n    })\n\n  // This is another subscription to the hub.\n  let unsubscribe_b =\n    reactive.subscribe(time_hub, fn(value) {\n      io.println(&quot;[B] | Current time: &quot; &lt;&gt; value)\n    })\n\n  // Just another client.\n  let unsubscribe_c =\n    reactive.subscribe(time_hub, fn(value) {\n      io.println(&quot;[C] | Current time: &quot; &lt;&gt; value)\n    })\n\n  // Notifies all subscribers of the hub that the current time has changed.\n  // This will call the provided function when the hub was created and broadcast the return value.\n  reactive.notify(time_hub)\n\n  // Unsubscribes from the hub.\n  unsubscribe_b()\n\n  // Notifies all subscribers again.\n  // If a real-time function is used, the current time will be updated.\n  reactive.notify(time_hub)\n\n  // Unsubscribe all subscribers.\n  unsubscribe_c()\n  unsubscribe_a()\n}\n```\n\n#### Output:\n\n&gt; [A] | Current time: 2024-05-21T16:30:00Z  \n&gt; [B] | Current time: 2024-05-21T16:30:00Z  \n&gt; [C] | Current time: 2024-05-21T16:30:00Z  \n&gt; [A] | Current time: 2024-05-21T16:30:00Z  \n&gt; [C] | Current time: 2024-05-21T16:30:00Z\n\n### Single topic-based Observer\n\nA simple topic-based observer implementation. Topics can be used to organize and\nfilter events.\n\n```gleam\nfn single_topic() {\n  use hub &lt;- topic.new()\n\n  // Creates a listener for the topics `&quot;a&quot;`, `&quot;ab&quot;` and `&quot;*&quot;`.\n  let unsubscribe_a =\n    topic.subscribe(hub, [&quot;a&quot;, &quot;ab&quot;, &quot;*&quot;], fn(value) {\n      io.println(&quot;[A] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // Creates a listener for the topics `&quot;b&quot;`, `&quot;ab&quot;` and `&quot;*&quot;`.\n  let unsubscribe_b =\n    topic.subscribe(hub, [&quot;b&quot;, &quot;ab&quot;, &quot;*&quot;], fn(value) {\n      io.println(&quot;[B] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // Creates a listener for the topics `&quot;c&quot;` and `&quot;*&quot;`.\n  let unsubscribe_c =\n    topic.subscribe(hub, [&quot;c&quot;, &quot;*&quot;], fn(value) {\n      io.println(&quot;[C] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // Notifies listeners with the topics `&quot;a&quot;` and `&quot;c&quot;`.\n  topic.notify(hub, [&quot;a&quot;, &quot;c&quot;], &quot;Hello A and C!&quot;)\n\n  // Notifies listeners with the topic `&quot;*&quot;`.\n  topic.notify(hub, [&quot;*&quot;], &quot;Hello all!&quot;)\n\n  // Notifies listeners with the topic `&quot;ab&quot;`.\n  topic.notify(hub, [&quot;ab&quot;], &quot;Hello AB!&quot;)\n\n  // Notifies listeners with the topics `&quot;a&quot;`, `&quot;ab&quot;`, `&quot;b&quot;`, `&quot;c&quot;` and `&quot;*&quot;`.\n  topic.notify(hub, [&quot;a&quot;, &quot;ab&quot;, &quot;b&quot;, &quot;c&quot;, &quot;*&quot;], &quot;Hello A, AB, B, C, and all!&quot;)\n\n  // Unsubscribe all listeners.\n  unsubscribe_a()\n  unsubscribe_b()\n  unsubscribe_c()\n}\n```\n\n#### Output:\n\n&gt; [A] | Received an event with value: Hello A and C!  \n&gt; [C] | Received an event with value: Hello A and C!  \n&gt; [A] | Received an event with value: Hello all!  \n&gt; [B] | Received an event with value: Hello all!  \n&gt; [C] | Received an event with value: Hello all!  \n&gt; [A] | Received an event with value: Hello AB!  \n&gt; [B] | Received an event with value: Hello AB!  \n&gt; [A] | Received an event with value: Hello A, AB, B, C, and all!  \n&gt; [B] | Received an event with value: Hello A, AB, B, C, and all!  \n&gt; [C] | Received an event with value: Hello A, AB, B, C, and all!\n\n### Multiple topic-based Observer\n\nExample of using multiple topics. This can be used for advanced filtering and\ngrouping of events. A possible example would be to propagate database changes to\nlisteners. In this case, the first topic would be the table name and the second\ntopics could be the column names.\n\n```gleam\nfn multiple_topics() {\n  use hub &lt;- topic.new2()\n\n  // Creates a listener for the `&quot;user&quot;` table and the columns `&quot;name&quot;` and `&quot;age&quot;`.\n  let unsubscribe_user_name_age =\n    topic.subscribe2(hub, [&quot;user&quot;], [&quot;name&quot;, &quot;age&quot;, &quot;*&quot;], fn(value) {\n      io.println(&quot;[User: name, age] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // Creates a listener for the `&quot;user&quot;` table and the columns `&quot;name&quot;`, `&quot;age&quot;` and `&quot;email&quot;`.\n  let unsubscribe_user_details =\n    topic.subscribe2(hub, [&quot;user&quot;], [&quot;name&quot;, &quot;age&quot;, &quot;email&quot;, &quot;*&quot;], fn(value) {\n      io.println(\n        &quot;[User: name, age, email] | Received an event with value: &quot; &lt;&gt; value,\n      )\n    })\n\n  // Update the user name\n  topic.notify2(hub, [&quot;user&quot;], [&quot;name&quot;], &quot;John Doe&quot;)\n\n  // Update the user email\n  topic.notify2(hub, [&quot;user&quot;], [&quot;email&quot;], &quot;john.doe@example.com&quot;)\n\n  // Add a new user\n  topic.notify2(hub, [&quot;user&quot;], [&quot;*&quot;], &quot;Hello World!&quot;)\n\n  unsubscribe_user_name_age()\n  unsubscribe_user_details()\n}\n```\n\n#### Output:\n\n&gt; [User: name, age] | Received an event with value: John Doe  \n&gt; [User: name, age, email] | Received an event with value: John Doe  \n&gt; [User: name, age, email] | Received an event with value:\n&gt; john.doe@example.com  \n&gt; [User: name, age] | Received an event with value: Hello World!  \n&gt; [User: name, age, email] | Received an event with value: Hello World!\n\n### Filtered Observer\n\nThis example demonstrates the usage of the filtered event_hub. It is an easy way\nto filter events based on a list of topics. They work in a similar way, but you\ncan use generics to filter by any type.\n\n```gleam\nfn filtered_observer() {\n  use hub &lt;- filtered.new3()\n\n  let unsubscribe_a =\n    filtered.subscribe3(hub, [0, 1], [&quot;A&quot;, &quot;*&quot;], [Ok(Nil)], fn(value) {\n      io.println(&quot;[A] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  let unsubscribe_b =\n    filtered.subscribe3(hub, [0, 2], [&quot;B&quot;, &quot;*&quot;], [Ok(Nil)], fn(value) {\n      io.println(&quot;[B] | Received an event with value: &quot; &lt;&gt; value)\n    })\n\n  // This should notify all subscribers.\n  filtered.notify3(hub, [0], [&quot;*&quot;], [Ok(Nil)], &quot;Hello all!&quot;)\n\n  // This should notify only subscriber A.\n  filtered.notify3(hub, [1], [&quot;A&quot;, &quot;*&quot;], [Ok(Nil)], &quot;Hello A&quot;)\n\n  // This should notify only subscriber B.\n  filtered.notify3(hub, [0, 1, 2], [&quot;B&quot;], [Ok(Nil)], &quot;Hello B&quot;)\n\n  // This should notify none.\n  filtered.notify3(hub, [0, 1, 2], [&quot;A&quot;, &quot;B&quot;, &quot;*&quot;], [Error(Nil)], &quot;Hello C&quot;)\n\n  unsubscribe_a()\n  unsubscribe_b()\n}\n```\n\n#### Output:\n\n&gt; [A] | Received an event with value: Hello all!  \n&gt; [B] | Received an event with value: Hello all!  \n&gt; [A] | Received an event with value: Hello A  \n&gt; [B] | Received an event with value: Hello B\n","url":"index.html"},{"doc":"event_hub","title":"event_hub","content":"# Changelog\n\n## 24.05.2024 - Version 1.0.1\n\n- Removed dependency on `gleam_erlang`.\n- Updated readme.\n\n## 22.05.2024 - Version 1.0.0\n\n- Initial release.\n","url":"changelog.html"},{"doc":"event_hub","title":"Callback","content":"pub type Callback(value_type) =\n  fn(value_type) -&gt; Nil\n Represents a callback function that gets called with a value when an event occurs.\n\nSynonyms:\nevent_hub.Callback\nevent_hub Callback","url":"event_hub.html#Callback"},{"doc":"event_hub","title":"Hub","content":"pub type Hub(value_type)\n Represents a hub for managing event subscriptions and notifications.\n\nSynonyms:\nevent_hub.Hub\nevent_hub Hub","url":"event_hub.html#Hub"},{"doc":"event_hub","title":"Unsubscribe","content":"pub type Unsubscribe =\n  fn() -&gt; Nil\n Represents an unsubscribe function that can be called to remove a previously added callback.\n\nSynonyms:\nevent_hub.Unsubscribe\nevent_hub Unsubscribe","url":"event_hub.html#Unsubscribe"},{"doc":"event_hub","title":"new","content":"pub fn new(in context: fn(Hub(a)) -&gt; b) -&gt; b\n Creates a new stateless observer hub, executes the given context with the hub, and stops the hub afterward.\n\n ## Parameters\n - `context`: A function that takes the created `Hub` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub\n \n pub fn example() {\n   event_hub.new(fn(hub) { event_hub.notify(hub, &quot;event&quot;) })\n }\n \nSynonyms:\nevent_hub.new\nevent_hub new","url":"event_hub.html#new"},{"doc":"event_hub","title":"notify","content":"pub fn notify(on hub: Hub(a), with value: a) -&gt; Nil\n Notifies all subscribers of the hub that an event has occurred with the given value.\n These notifications occur in parallel but `notify` waits for all of them to complete.\n\n ## Parameters\n - `hub`: The `Hub` to notify.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub\n\n pub fn example(hub: event_hub.Hub(String)) {\n   event_hub.notify(hub, &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub.notify\nevent_hub notify","url":"event_hub.html#notify"},{"doc":"event_hub","title":"subscribe","content":"pub fn subscribe(\n  on hub: Hub(a),\n  with callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Adds a callback to the hub and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub` to add the callback to.\n - `callback`: The callback function to add.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub\n \n pub fn example(hub: event_hub.Hub(String)) {\n   let unsubscribe =\n     event_hub.subscribe(hub, fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub.subscribe\nevent_hub subscribe","url":"event_hub.html#subscribe"},{"doc":"event_hub","title":"event_hub","content":" The `event_hub` module provides a way to manage and notify subscribers about events.\n It supports stateless observers, allowing functions to be registered and invoked in parallel when an event occurs.\n\n ## Examples\n\n ### Simple Observer\n ```gleam\n import gleam/io\n import event_hub\n \n pub fn main() {\n   use hub &lt;- event_hub.new()\n \n   let unsubscribe =\n     event_hub.subscribe(hub, fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   event_hub.notify(hub, &quot;Hello, world!&quot;)\n   unsubscribe()\n   event_hub.notify(hub, &quot;This won&#39;t be received&quot;)\n }\n ```","url":"event_hub.html"},{"doc":"event_hub/filtered","title":"Hub","content":"pub opaque type Hub(value_type, topic_type)\n Represents a hub for managing event subscriptions and notifications based on topics.\n\nSynonyms:\nevent_hub/filtered.Hub\nevent_hub/filtered Hub","url":"event_hub/filtered.html#Hub"},{"doc":"event_hub/filtered","title":"Hub2","content":"pub opaque type Hub2(value_type, topic_type1, topic_type2)\n Represents a hub for managing event subscriptions and notifications based on two levels of topics.\n\nSynonyms:\nevent_hub/filtered.Hub2\nevent_hub/filtered Hub2","url":"event_hub/filtered.html#Hub2"},{"doc":"event_hub/filtered","title":"Hub3","content":"pub opaque type Hub3(\n  value_type,\n  topic_type1,\n  topic_type2,\n  topic_type3,\n)\n Represents a hub for managing event subscriptions and notifications based on three levels of topics.\n\nSynonyms:\nevent_hub/filtered.Hub3\nevent_hub/filtered Hub3","url":"event_hub/filtered.html#Hub3"},{"doc":"event_hub/filtered","title":"Hub4","content":"pub opaque type Hub4(\n  value_type,\n  topic_type1,\n  topic_type2,\n  topic_type3,\n  topic_type4,\n)\n Represents a hub for managing event subscriptions and notifications based on four levels of topics.\n\nSynonyms:\nevent_hub/filtered.Hub4\nevent_hub/filtered Hub4","url":"event_hub/filtered.html#Hub4"},{"doc":"event_hub/filtered","title":"new","content":"pub fn new(in context: fn(Hub(a, b)) -&gt; c) -&gt; c\n Creates a new filtered event hub, executes the given context with the hub.\n If you only want to use topics of the type `String`, you should use the `event_hub/topic` module instead.\n\n ## Parameters\n - `context`: A function that takes the created `Hub` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example() {\n   filtered.new(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/filtered.new\nevent_hub/filtered new","url":"event_hub/filtered.html#new"},{"doc":"event_hub/filtered","title":"new2","content":"pub fn new2(in context: fn(Hub2(a, b, c)) -&gt; d) -&gt; d\n Creates a new filtered event hub with two levels of topics.\n If you only want to use topics of the type `String`, you should use the `event_hub/topic` module instead.\n\n ## Parameters\n - `context`: A function that takes the created `Hub2` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example() {\n   filtered.new2(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/filtered.new2\nevent_hub/filtered new2","url":"event_hub/filtered.html#new2"},{"doc":"event_hub/filtered","title":"new3","content":"pub fn new3(in context: fn(Hub3(a, b, c, d)) -&gt; e) -&gt; e\n Creates a new filtered event hub with three levels of topics.\n If you only want to use topics of the type `String`, you should use the `event_hub/topic` module instead.\n\n ## Parameters\n - `context`: A function that takes the created `Hub3` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example() {\n   filtered.new3(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/filtered.new3\nevent_hub/filtered new3","url":"event_hub/filtered.html#new3"},{"doc":"event_hub/filtered","title":"new4","content":"pub fn new4(in context: fn(Hub4(a, b, c, d, e)) -&gt; f) -&gt; f\n Creates a new filtered event hub with four levels of topics.\n If you only want to use topics of the type `String`, you should use the `event_hub/topic` module instead.\n\n ## Parameters\n - `context`: A function that takes the created `Hub4` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example() {\n   filtered.new4(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/filtered.new4\nevent_hub/filtered new4","url":"event_hub/filtered.html#new4"},{"doc":"event_hub/filtered","title":"notify","content":"pub fn notify(\n  on hub: Hub(a, b),\n  with topics: List(b),\n  and value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with the given topics and value.\n These notifications occur in parallel but `notify` waits for all of them to complete.\n\n ## Parameters\n - `hub`: The `Hub` to notify.\n - `topics`: The list of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example(hub: filtered.Hub(String, String)) {\n   filtered.notify(hub, [&quot;topic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/filtered.notify\nevent_hub/filtered notify","url":"event_hub/filtered.html#notify"},{"doc":"event_hub/filtered","title":"notify2","content":"pub fn notify2(\n  hub: Hub2(a, b, c),\n  topics1: List(b),\n  topics2: List(c),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with two levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub2` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub2(String, String, String)) {\n   filtered.notify2(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/filtered.notify2\nevent_hub/filtered notify2","url":"event_hub/filtered.html#notify2"},{"doc":"event_hub/filtered","title":"notify3","content":"pub fn notify3(\n  hub: Hub3(a, b, c, d),\n  topics1: List(b),\n  topics2: List(c),\n  topics3: List(d),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with three levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub3` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `topics3`: The third level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/filtered\n\n pub fn example(hub: filtered.Hub3(String, String, String, String)) {\n   filtered.notify3(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], [&quot;subsubtopic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/filtered.notify3\nevent_hub/filtered notify3","url":"event_hub/filtered.html#notify3"},{"doc":"event_hub/filtered","title":"notify4","content":"pub fn notify4(\n  hub: Hub4(a, b, c, d, e),\n  topics1: List(b),\n  topics2: List(c),\n  topics3: List(d),\n  topics4: List(e),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with four levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub4` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `topics3`: The third level of topics associated with the event.\n - `topics4`: The fourth level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub4(String, String, String, String, String)) {\n   filtered.notify4(\n     hub,\n     [&quot;topic1&quot;],\n     [&quot;subtopic1&quot;],\n     [&quot;subsubtopic1&quot;],\n     [&quot;subsubsubtopic1&quot;],\n     &quot;event&quot;,\n   )\n }\n \nSynonyms:\nevent_hub/filtered.notify4\nevent_hub/filtered notify4","url":"event_hub/filtered.html#notify4"},{"doc":"event_hub/filtered","title":"subscribe","content":"pub fn subscribe(\n  on hub: Hub(a, b),\n  with topics: List(b),\n  and callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific topics and returns an unsubscribe function.\n The callback will be invoked only if the event topics match the subscription topics.\n\n ## Parameters\n - `hub`: The `Hub` to add the callback to.\n - `topics`: The list of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub(String, String)) {\n   let unsubscribe =\n     filtered.subscribe(hub, [&quot;topic1&quot;], fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/filtered.subscribe\nevent_hub/filtered subscribe","url":"event_hub/filtered.html#subscribe"},{"doc":"event_hub/filtered","title":"subscribe2","content":"pub fn subscribe2(\n  hub: Hub2(a, b, c),\n  topics1: List(b),\n  topics2: List(c),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific topics in a two-level hierarchy and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub2` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub2(String, String, String)) {\n   let unsubscribe =\n     filtered.subscribe2(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/filtered.subscribe2\nevent_hub/filtered subscribe2","url":"event_hub/filtered.html#subscribe2"},{"doc":"event_hub/filtered","title":"subscribe3","content":"pub fn subscribe3(\n  hub: Hub3(a, b, c, d),\n  topics1: List(b),\n  topics2: List(c),\n  topics3: List(d),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific topics in a three-level hierarchy and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub3` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `topics3`: The third level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub3(String, String, String, String)) {\n   let unsubscribe =\n     filtered.subscribe3(\n       hub,\n       [&quot;topic1&quot;],\n       [&quot;subtopic1&quot;],\n       [&quot;subsubtopic1&quot;],\n       fn(value) { io.println(&quot;Received value: &quot; &lt;&gt; value) },\n     )\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/filtered.subscribe3\nevent_hub/filtered subscribe3","url":"event_hub/filtered.html#subscribe3"},{"doc":"event_hub/filtered","title":"subscribe4","content":"pub fn subscribe4(\n  hub: Hub4(a, b, c, d, e),\n  topics1: List(b),\n  topics2: List(c),\n  topics3: List(d),\n  topics4: List(e),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific topics in a four-level hierarchy and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub4` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `topics3`: The third level of topics to subscribe to.\n - `topics4`: The fourth level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/filtered\n \n pub fn example(hub: filtered.Hub4(String, String, String, String, String)) {\n   let unsubscribe =\n     filtered.subscribe4(\n       hub,\n       [&quot;topic1&quot;],\n       [&quot;subtopic1&quot;],\n       [&quot;subsubtopic1&quot;],\n       [&quot;subsubsubtopic1&quot;],\n       fn(value) { io.println(&quot;Received value: &quot; &lt;&gt; value) },\n     )\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/filtered.subscribe4\nevent_hub/filtered subscribe4","url":"event_hub/filtered.html#subscribe4"},{"doc":"event_hub/filtered","title":"event_hub/filtered","content":" The `filtered` module provides a way to manage and notify subscribers about events based on specific topics.\n It supports creating observers that can filter events using topics, allowing more fine-grained control over event handling.\n If you only want to use topics of the type `String`, you should use the `event_hub/topic` module instead.\n \n ## Examples\n\n ### Filtered Observer\n ```gleam\n import gleam/io\n import event_hub/filtered\n \n pub fn main() {\n   use hub &lt;- filtered.new()\n \n   let unsubscribe_a =\n     filtered.subscribe(hub, [Ok(&quot;topic1&quot;)], fn(value) {\n       io.println(&quot;A received: &quot; &lt;&gt; value)\n     })\n \n   let unsubscribe_b =\n     filtered.subscribe(hub, [Error(&quot;topic2&quot;)], fn(value) {\n       io.println(&quot;B received: &quot; &lt;&gt; value)\n     })\n \n   filtered.notify(hub, [Ok(&quot;topic1&quot;)], &quot;Message for topic1&quot;)\n   filtered.notify(hub, [Error(&quot;topic2&quot;)], &quot;Message for topic2&quot;)\n \n   unsubscribe_a()\n   unsubscribe_b()\n }\n ```","url":"event_hub/filtered.html"},{"doc":"event_hub/reactive","title":"Hub","content":"pub opaque type Hub(value_type)\n Represents a hub for managing event subscriptions and notifications based on dynamic state.\n\nSynonyms:\nevent_hub/reactive.Hub\nevent_hub/reactive Hub","url":"event_hub/reactive.html#Hub"},{"doc":"event_hub/reactive","title":"State","content":"pub type State(value_type) =\n  fn() -&gt; value_type\n Represents a function that returns the current state value.\n\nSynonyms:\nevent_hub/reactive.State\nevent_hub/reactive State","url":"event_hub/reactive.html#State"},{"doc":"event_hub/reactive","title":"new","content":"pub fn new(\n  with state: fn() -&gt; a,\n  in context: fn(Hub(a)) -&gt; b,\n) -&gt; b\n Creates a new reactive observer hub, executes the given context with the hub.\n\n ## Parameters\n - `state`: A function that returns the current state value.\n - `context`: A function that takes the created `Hub` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/reactive\n\n pub fn example() {\n   let get_state = fn() { &quot;current state&quot; }\n\n   reactive.new(get_state, fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/reactive.new\nevent_hub/reactive new","url":"event_hub/reactive.html#new"},{"doc":"event_hub/reactive","title":"notify","content":"pub fn notify(on hub: Hub(a)) -&gt; a\n Notifies subscribers of the hub about an event with the current state value.\n These notifications occur in parallel but `notify` waits for all of them to complete.\n\n ## Parameters\n - `hub`: The `Hub` to notify.\n\n ## Returns\n The current state value.\n\n ## Example\n \n import event_hub/reactive\n\n pub fn example(hub: reactive.Hub(String)) {\n   let value = reactive.notify(hub)\n }\n \nSynonyms:\nevent_hub/reactive.notify\nevent_hub/reactive notify","url":"event_hub/reactive.html#notify"},{"doc":"event_hub/reactive","title":"subscribe","content":"pub fn subscribe(\n  on hub: Hub(a),\n  with callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to state changes and returns an unsubscribe function.\n The callback will be invoked with the current state value when `notify` is called.\n\n ## Parameters\n - `hub`: The `Hub` to add the callback to.\n - `callback`: The callback function to invoke with the current state value.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/reactive\n \n pub fn example(hub: reactive.Hub(String)) {\n   let unsubscribe =\n     reactive.subscribe(hub, fn(value) {\n       io.println(&quot;Received state: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/reactive.subscribe\nevent_hub/reactive subscribe","url":"event_hub/reactive.html#subscribe"},{"doc":"event_hub/reactive","title":"event_hub/reactive","content":" The `reactive` module provides a way to manage and notify subscribers about events based on a state that can be\n dynamically retrieved. It supports creating observers that can react to changes in state, which can be useful for\n broadcasting things that constantly change like the current time or database updates.\n\n ## Examples\n\n ### Reactive Observer\n ```gleam\n import gleam/io\n import event_hub/reactive\n \n pub fn main() {\n   let get_time = fn() { &quot;2024-05-21T16:30:00Z&quot; }\n \n   use hub &lt;- reactive.new(get_time)\n   let unsubscribe =\n     reactive.subscribe(hub, fn(value) { io.println(&quot;Current time: &quot; &lt;&gt; value) })\n \n   reactive.notify(hub)\n \n   unsubscribe()\n   reactive.notify(hub)\n }\n ```","url":"event_hub/reactive.html"},{"doc":"event_hub/stateful","title":"Hub","content":"pub type Hub(value_type)\n Represents a hub for managing event subscriptions and notifications with a mutable state.\n\nSynonyms:\nevent_hub/stateful.Hub\nevent_hub/stateful Hub","url":"event_hub/stateful.html#Hub"},{"doc":"event_hub/stateful","title":"new","content":"pub fn new(with value: a, in context: fn(Hub(a)) -&gt; b) -&gt; b\n Creates a new stateful observer hub with an initial state, executes the given context with the hub, and stops the hub afterward.\n\n ## Parameters\n - `value`: The initial state value.\n - `context`: A function that takes the created `Hub` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/stateful\n\n pub fn example() {\n   stateful.new(&quot;initial state&quot;, fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/stateful.new\nevent_hub/stateful new","url":"event_hub/stateful.html#new"},{"doc":"event_hub/stateful","title":"notify","content":"pub fn notify(on hub: Hub(a), with value: a) -&gt; Nil\n Notifies subscribers of the hub about an event with a new state value.\n These notifications occur in parallel but `notify` waits for all of them to complete.\n\n ## Parameters\n - `hub`: The `Hub` to notify.\n - `value`: The new state value.\n\n ## Example\n \n import event_hub/stateful\n\n pub fn example(hub: stateful.Hub(String)) {\n   stateful.notify(hub, &quot;new state&quot;)\n }\n \nSynonyms:\nevent_hub/stateful.notify\nevent_hub/stateful notify","url":"event_hub/stateful.html#notify"},{"doc":"event_hub/stateful","title":"state","content":"pub fn state(of hub: Hub(a)) -&gt; a\n Retrieves the current state of the hub.\n\n ## Parameters\n - `hub`: The `Hub` to retrieve the state from.\n\n ## Returns\n The current state value.\n\n ## Example\n \n import event_hub/stateful\n\n pub fn example(hub: stateful.Hub(String)) {\n   let current_state = stateful.state(hub)\n }\n \nSynonyms:\nevent_hub/stateful.state\nevent_hub/stateful state","url":"event_hub/stateful.html#state"},{"doc":"event_hub/stateful","title":"subscribe","content":"pub fn subscribe(\n  on hub: Hub(a),\n  should notify_current_state: Bool,\n  with callback: fn(a) -&gt; Nil,\n) -&gt; #(a, fn() -&gt; Nil)\n Subscribes to state changes and returns the current state and an unsubscribe function.\n The callback will be invoked with the current state value when `notify` is called.\n If `notify_current_state` is `True`, the callback will be immediately invoked with the current state.\n\n ## Parameters\n - `hub`: The `Hub` to add the callback to.\n - `notify_current_state`: Whether to immediately invoke the callback with the current state.\n - `callback`: The callback function to invoke with the state value.\n\n ## Returns\n A tuple containing the current state value and an `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/stateful\n \n pub fn example(hub: stateful.Hub(String)) {\n   let #(current_state, unsubscribe) =\n     stateful.subscribe(hub, True, fn(value) {\n       io.println(&quot;Received state: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/stateful.subscribe\nevent_hub/stateful subscribe","url":"event_hub/stateful.html#subscribe"},{"doc":"event_hub/stateful","title":"event_hub/stateful","content":" The `stateful` module provides a way to manage and notify subscribers about events based on a mutable state.\n It supports creating observers that can maintain and update state, which can be useful for handling stateful events\n in an application.\n\n ## Examples\n\n ### Stateful Observer\n ```gleam\n import gleam/io\n import event_hub/stateful\n \n pub fn main() {\n   use hub &lt;- stateful.new(&quot;initial state&quot;)\n   let #(current_state, unsubscribe) =\n     stateful.subscribe(hub, True, fn(value) {\n       io.println(&quot;Received initial state: &quot; &lt;&gt; value)\n     })\n \n   io.println(&quot;Current state: &quot; &lt;&gt; current_state)\n \n   stateful.notify(hub, &quot;new state&quot;)\n \n   unsubscribe()\n   stateful.notify(hub, &quot;final state&quot;)\n }\n ```","url":"event_hub/stateful.html"},{"doc":"event_hub/topic","title":"Hub","content":"pub opaque type Hub(value_type)\n Represents a hub for managing event subscriptions and notifications based on a single level of topics.\n\nSynonyms:\nevent_hub/topic.Hub\nevent_hub/topic Hub","url":"event_hub/topic.html#Hub"},{"doc":"event_hub/topic","title":"Hub2","content":"pub opaque type Hub2(value_type)\n Represents a hub for managing event subscriptions and notifications based on two levels of topics.\n\nSynonyms:\nevent_hub/topic.Hub2\nevent_hub/topic Hub2","url":"event_hub/topic.html#Hub2"},{"doc":"event_hub/topic","title":"Hub3","content":"pub opaque type Hub3(value_type)\n Represents a hub for managing event subscriptions and notifications based on three levels of topics.\n\nSynonyms:\nevent_hub/topic.Hub3\nevent_hub/topic Hub3","url":"event_hub/topic.html#Hub3"},{"doc":"event_hub/topic","title":"Hub4","content":"pub opaque type Hub4(value_type)\n Represents a hub for managing event subscriptions and notifications based on four levels of topics.\n\nSynonyms:\nevent_hub/topic.Hub4\nevent_hub/topic Hub4","url":"event_hub/topic.html#Hub4"},{"doc":"event_hub/topic","title":"HubN","content":"pub type HubN(value_type)\n Represents a hub for managing event subscriptions and notifications based on hierarchical topics.\n\nSynonyms:\nevent_hub/topic.HubN\nevent_hub/topic HubN","url":"event_hub/topic.html#HubN"},{"doc":"event_hub/topic","title":"new","content":"pub fn new(in context: fn(Hub(a)) -&gt; b) -&gt; b\n Creates a new single-level topic-based event hub, executes the given context with the hub.\n\n ## Parameters\n - `context`: A function that takes the created `Hub` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example() {\n   topic.new(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/topic.new\nevent_hub/topic new","url":"event_hub/topic.html#new"},{"doc":"event_hub/topic","title":"new2","content":"pub fn new2(in context: fn(Hub2(a)) -&gt; b) -&gt; b\n Creates a new two-level topic-based event hub, executes the given context with the hub.\n\n ## Parameters\n - `context`: A function that takes the created `Hub2` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example() {\n   topic.new2(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/topic.new2\nevent_hub/topic new2","url":"event_hub/topic.html#new2"},{"doc":"event_hub/topic","title":"new3","content":"pub fn new3(in context: fn(Hub3(a)) -&gt; b) -&gt; b\n Creates a new three-level topic-based event hub, executes the given context with the hub.\n\n ## Parameters\n - `context`: A function that takes the created `Hub3` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example() {\n   topic.new3(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/topic.new3\nevent_hub/topic new3","url":"event_hub/topic.html#new3"},{"doc":"event_hub/topic","title":"new4","content":"pub fn new4(in context: fn(Hub4(a)) -&gt; b) -&gt; b\n Creates a new four-level topic-based event hub, executes the given context with the hub.\n\n ## Parameters\n - `context`: A function that takes the created `Hub4` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example() {\n   topic.new4(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/topic.new4\nevent_hub/topic new4","url":"event_hub/topic.html#new4"},{"doc":"event_hub/topic","title":"new_n","content":"pub fn new_n(in context: fn(HubN(a)) -&gt; b) -&gt; b\n Creates a new topic-based event hub, executes the given context with the hub, and stops the hub afterward.\n\n ## Parameters\n - `context`: A function that takes the created `HubN` and returns a result.\n\n ## Returns\n The result of executing the context function.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example() {\n   topic.new(fn(hub) {\n     // Use the hub\n     Nil\n   })\n }\n \nSynonyms:\nevent_hub/topic.new_n\nevent_hub/topic new_n","url":"event_hub/topic.html#new_n"},{"doc":"event_hub/topic","title":"notify","content":"pub fn notify(\n  on hub: Hub(a),\n  with topics: List(String),\n  and value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with the given single-level topics and value.\n\n ## Parameters\n - `hub`: The `Hub` to notify.\n - `topics`: The list of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example(hub: topic.Hub(String)) {\n   topic.notify(hub, [&quot;topic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/topic.notify\nevent_hub/topic notify","url":"event_hub/topic.html#notify"},{"doc":"event_hub/topic","title":"notify2","content":"pub fn notify2(\n  hub: Hub2(a),\n  topics1: List(String),\n  topics2: List(String),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with two levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub2` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example(hub: topic.Hub2(String)) {\n   topic.notify2(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/topic.notify2\nevent_hub/topic notify2","url":"event_hub/topic.html#notify2"},{"doc":"event_hub/topic","title":"notify3","content":"pub fn notify3(\n  hub: Hub3(a),\n  topics1: List(String),\n  topics2: List(String),\n  topics3: List(String),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with three levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub3` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `topics3`: The third level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example(hub: topic.Hub3(String)) {\n   topic.notify3(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], [&quot;subsubtopic1&quot;], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/topic.notify3\nevent_hub/topic notify3","url":"event_hub/topic.html#notify3"},{"doc":"event_hub/topic","title":"notify4","content":"pub fn notify4(\n  hub: Hub4(a),\n  topics1: List(String),\n  topics2: List(String),\n  topics3: List(String),\n  topics4: List(String),\n  value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with four levels of topics and a value.\n\n ## Parameters\n - `hub`: The `Hub4` to notify.\n - `topics1`: The first level of topics associated with the event.\n - `topics2`: The second level of topics associated with the event.\n - `topics3`: The third level of topics associated with the event.\n - `topics4`: The fourth level of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/topic\n \n pub fn example(hub: topic.Hub4(String)) {\n   topic.notify4(\n     hub,\n     [&quot;topic1&quot;],\n     [&quot;subtopic1&quot;],\n     [&quot;subsubtopic1&quot;],\n     [&quot;subsubsubtopic1&quot;],\n     &quot;event&quot;,\n   )\n }\n \nSynonyms:\nevent_hub/topic.notify4\nevent_hub/topic notify4","url":"event_hub/topic.html#notify4"},{"doc":"event_hub/topic","title":"notify_n","content":"pub fn notify_n(\n  on hub: HubN(a),\n  with topics: List(List(String)),\n  and value: a,\n) -&gt; Nil\n Notifies subscribers of the hub about an event with the given topics and value.\n These notifications occur in parallel but `notify_n` waits for all of them to complete.\n\n ## Parameters\n - `hub`: The `HubN` to notify.\n - `topics`: The list of topics associated with the event.\n - `value`: The value to send to all subscribers.\n\n ## Example\n \n import event_hub/topic\n\n pub fn example(hub: topic.HubN(String)) {\n   topic.notify_n(hub, [[&quot;topic1&quot;]], &quot;event&quot;)\n }\n \nSynonyms:\nevent_hub/topic.notify_n\nevent_hub/topic notify_n","url":"event_hub/topic.html#notify_n"},{"doc":"event_hub/topic","title":"subscribe","content":"pub fn subscribe(\n  on hub: Hub(a),\n  with topics: List(String),\n  and callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific single-level topics and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub` to add the callback to.\n - `topics`: The list of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/topic\n \n pub fn example(hub: topic.Hub(String)) {\n   let unsubscribe =\n     topic.subscribe(hub, [&quot;topic1&quot;], fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/topic.subscribe\nevent_hub/topic subscribe","url":"event_hub/topic.html#subscribe"},{"doc":"event_hub/topic","title":"subscribe2","content":"pub fn subscribe2(\n  hub: Hub2(a),\n  topics1: List(String),\n  topics2: List(String),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific two-level topics and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub2` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import event_hub/topic\n import gleam/io\n \n pub fn example(hub: topic.Hub2(String)) {\n   let unsubscribe =\n     topic.subscribe2(hub, [&quot;topic1&quot;], [&quot;subtopic1&quot;], fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/topic.subscribe2\nevent_hub/topic subscribe2","url":"event_hub/topic.html#subscribe2"},{"doc":"event_hub/topic","title":"subscribe3","content":"pub fn subscribe3(\n  hub: Hub3(a),\n  topics1: List(String),\n  topics2: List(String),\n  topics3: List(String),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific three-level topics and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub3` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `topics3`: The third level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/topic\n \n pub fn example(hub: topic.Hub3(String)) {\n   let unsubscribe =\n     topic.subscribe3(\n       hub,\n       [&quot;topic1&quot;],\n       [&quot;subtopic1&quot;],\n       [&quot;subsubtopic1&quot;],\n       fn(value) { io.println(&quot;Received value: &quot; &lt;&gt; value) },\n     )\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/topic.subscribe3\nevent_hub/topic subscribe3","url":"event_hub/topic.html#subscribe3"},{"doc":"event_hub/topic","title":"subscribe4","content":"pub fn subscribe4(\n  hub: Hub4(a),\n  topics1: List(String),\n  topics2: List(String),\n  topics3: List(String),\n  topics4: List(String),\n  callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific four-level topics and returns an unsubscribe function.\n\n ## Parameters\n - `hub`: The `Hub4` to add the callback to.\n - `topics1`: The first level of topics to subscribe to.\n - `topics2`: The second level of topics to subscribe to.\n - `topics3`: The third level of topics to subscribe to.\n - `topics4`: The fourth level of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/topic\n \n pub fn example(hub: topic.Hub4(String)) {\n   let unsubscribe =\n     topic.subscribe4(\n       hub,\n       [&quot;topic1&quot;],\n       [&quot;subtopic1&quot;],\n       [&quot;subsubtopic1&quot;],\n       [&quot;subsubsubtopic1&quot;],\n       fn(value) { io.println(&quot;Received value: &quot; &lt;&gt; value) },\n     )\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/topic.subscribe4\nevent_hub/topic subscribe4","url":"event_hub/topic.html#subscribe4"},{"doc":"event_hub/topic","title":"subscribe_n","content":"pub fn subscribe_n(\n  on hub: HubN(a),\n  with topics: List(List(String)),\n  and callback: fn(a) -&gt; Nil,\n) -&gt; fn() -&gt; Nil\n Subscribes to specific topics and returns an unsubscribe function.\n The callback will be invoked only if the event topics match the subscription topics.\n\n ## Parameters\n - `hub`: The `HubN` to add the callback to.\n - `topics`: The list of topics to subscribe to.\n - `callback`: The callback function to invoke when an event with matching topics occurs.\n\n ## Returns\n An `Unsubscribe` function that can be called to remove the callback.\n\n ## Example\n \n import gleam/io\n import event_hub/topic\n \n pub fn example(hub: topic.HubN(String)) {\n   let unsubscribe =\n     topic.subscribe_n(hub, [[&quot;topic1&quot;]], fn(value) {\n       io.println(&quot;Received value: &quot; &lt;&gt; value)\n     })\n \n   // To unsubscribe\n   unsubscribe()\n }\n \nSynonyms:\nevent_hub/topic.subscribe_n\nevent_hub/topic subscribe_n","url":"event_hub/topic.html#subscribe_n"},{"doc":"event_hub/topic","title":"event_hub/topic","content":" The `topic` module provides a way to manage and notify subscribers about events based on hierarchical topics.\n It supports creating observers that can filter events using one or more levels of topics, allowing more fine-grained\n control over event handling.\n\n ## Examples\n\n ### Single-Level Topic-Based Observer\n ```gleam\n import gleam/io\n import event_hub/topic\n \n pub fn main() {\n   use hub &lt;- topic.new()\n   let unsubscribe_a =\n     topic.subscribe(hub, [&quot;a&quot;], fn(value) {\n       io.println(&quot;A received: &quot; &lt;&gt; value)\n     })\n \n   let unsubscribe_b =\n     topic.subscribe(hub, [&quot;b&quot;], fn(value) {\n       io.println(&quot;B received: &quot; &lt;&gt; value)\n     })\n \n   topic.notify(hub, [&quot;a&quot;], &quot;Message for A&quot;)\n   topic.notify(hub, [&quot;b&quot;], &quot;Message for B&quot;)\n \n   unsubscribe_a()\n   unsubscribe_b()\n }\n ```","url":"event_hub/topic.html"}]);